package App::bif::Version;
$App::bif::Version::VERSION = '0.1.0_3';
$App::bif::Version::COMMIT = 'bdd365f87d37d37339e92d14014e01ac05a8e43a';
$App::bif::Version::BRANCH = 'devel';
$App::bif::Version::DATE = '2014-04-06 20:22:52';
1;
